<!-- toc -->

# 1、 简介

## 1.1、 适用范围

TapDB提供一套SDK，游戏开发者可以将其集成到游戏中。系统会收集用户数据，并进行分析，最终形成数据报表，帮助游戏开发者分析用户行为并优化游戏。

目前SDK适用于Android 4.0及以上的系统。

## 1.2、 名词解释

名词 | 含义
| ------ | ------ |
设备 | 安装了游戏的设备
分包渠道 | 标识游戏安装包的渠道，需要在代码中设置，可作为控制后台的过滤条件
用户 | 对应一个用户账户，需要一个唯一的标识符
付费 | 用户使用真实货币换取游戏虚拟币或游戏道具

# 2、 使用方法

## 2.1、 申请应用

在TapDB控制台中注册一个游戏，获得游戏对应的APP ID。这是一个长度为16的字符串。iOS和Android可共用一个APP ID。

## 2.2、向工程中导入SDK

在TapDB网站上下载最新的SDK，其中包含一个库文件libTapDB-xxx.aar（强烈建议使用最新版本）。将该库文件加入到项目依赖库中。

## 2.3、添加权限
```
    <!--必选权限-->
    <uses-permission android:name="android.permission.INTERNET" />
    <uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
    <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>

    <!--强烈建议权限  sdk获取IMEI时会需要此权限;获取不到不影响功能正常使用，IMEI用于辅助数据分析，使统计结果更加精确--> 
    <uses-permission android:name="android.permission.READ_PHONE_STATE" />
```
    

## 2.4、初始化
初始化TapDB SDK，调用这个接口是使用其它接口的先决条件，需要尽早调用。一般建议在游戏的主Activity中调用，如果多次调用，只有第一次调用视为有效。

```
public static void init(Context context, String appId, String appVersion, String channel)
public static void init(Context context, String appId, String channel,String appVersion,JSONObject properties)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
context | 否 | 当前应用的Context对象。一般使用MainActivity对象
appId | 否 | 在控制台注册得到到的APP ID
channel | 是 | 长度大于0并小于等于256。分包渠道。1.2.名词解释中有介绍
appVersion | 是 | 为空的话,会获取AndroidManifest.xml中versionCode作为appVersion
properties | 否 | 自定义属性


# 3、 常用接口说明

## 3.0.1、 记录一下用户（当用户登陆时调用）

记录一个用户，当用户登陆时调用。

```
public static void setUser(String userId)

public static void setUser(String userId，JSONObject properties)
```
字段 | 可为空 | 说明
| ------ | ------ | ------ |
userId | 否 | 长度大于0并小于等于256。只能包含数字、大小写字母、下划线(_)、横线(-)，用户ID。不同用户需要保证ID的唯一性
properties | 否 | 自定义属性

## 3.0.2、 修改数据发送的域名（有特殊需要时调用，调用必须位于初始化之前）
```
public static void setHost(String host)
```

## 3.0.3、 修改自定义事件数据发送的域名（有特殊需要时调用，调用必须位于初始化之前）
说明：后续不再支持，不建议使用
```
public static void setCustomEventHost(String customEventHost)
```

## 3.0.4、 上报用户名称

```
public static void setName(String name)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
name | 否 | 长度大于0并小于等于256。用户名

## 3.0.5、 上报用户等级
```
public static void setLevel(int level)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
level | 否 | 大于等于0。用户等级

## 3.0.6、 上报用户所在服务器（用户登录或切换服务器时调用）

```
public static void setServer(String server)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
server | 否 | 长度大于0并小于等于256。用户所在服务器

## 3.0.7、 上报充值成功（必须在充值成功时调用）

```
public static void onCharge(String orderId, String product, long amount, String currencyType, String payment)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
orderId | 是 | 长度大于0并小于等于256。订单ID。传递订单ID可进行排重，防止计算多次
product | 是 | 长度大于0并小于等于256。商品名称
amount | 否 | 大于0并小于等于100000000000。充值金额。单位分，即无论什么币种，都需要乘以100
currencyType | 是 | 货币类型。国际通行三字母表示法，为空时默认CNY。参考：人民币 CNY，美元 USD；欧元 EUR
payment | 是 | 长度大于0并小于等于256。充值渠道

常见货币类型的格式参考<a target="_blank" href="https://www.tapdb.com/docs/zh_CN/features/exchangeRate.html">汇率表</a>

## 3.0.8、 上报自定义事件

```
public static void track(String eventName, JSONObject properties)

//该方法后续会被弃用，不建议使用
public static void onEvent(String eventName, JSONObject properties)
```

字段 | 可为空 | 说明
| ------ | ------ | ------ |
eventName | 否 | 在控制台中配置得到的事件名称
properties | 是 | 事件属性。需要和控制台的配置匹配。值需要是长度大于0并小于等于256的字符串或绝对值小于1E11的浮点数

## 3.0.9、 退出登录
退出登录时，需调用下面的下面，从而清除掉缓存对应userId用户的数据
```
public static void clearUser()
```

## 3.1.0 事件主体操作（用户、设备）
```
//设备初始化操作
public static void deviceInitialize(final JSONObject properties)

//设备更新操作
public static void deviceUpdate(final JSONObject properties)

//设备添加操作
public static void deviceAdd(final JSONObject properties)

//用户初始化操作
public static void userInitialize(final JSONObject properties)

//用户更新操作
public static void userUpdate(final JSONObject properties)

//用户添加操作
public static void userAdd(final JSONObject properties)
```

## 3.1.1 自定义属性
用户可以注册自定义属性，包括静态固定属性和动态计算属性，附带在TapDB事件中上传。
```
//添加静态事件属性，每个事件都将会发送
public static void registerStaticProperties(final JSONObject staticProperties)

// 删除添加的静态事件属性
public static void clearStaticProperties()

/// 删除添加的某个静态事件属性
public static void unregisterStaticProperty(final String propertyName)

//添加动态事件属性，每次发送事件会调用
public static void registerDynamicProperties(final TapDBDataDynamicProperties dynamicProperties)
```


## 3.1.2、 上报应用时长

初始化TapDB sdk之后，sdk会自动在应用置于后台时上报当前时长，并以此次上报作为起始点，下次若置于后台，则上报此时间段的时长。事件名为：duration；时长属性为：play_game

**说明：之前版本，需要应用Activity生命周期回调函数onResume方法里调用TapDB.onResume方法，onStop方法里调用TapDB.onStop方法**



# 4、 第三方设备ID
**说明：设备ID会使数据统计更加精确，建议添加，TapDB SDK支持获取OAID的设备ID以及数美设备ID(需要手动添加oaid的sdk以及数美sdk)，获取到的设备id,会上报到服务器，辅助数据分析，使统计结果更加精准，如果不需要可以跳过该步骤**
## 4.1、 OAID sdk获取设备ID

TapDB SDK当前支持oaid sdk 1.0.5~1.0.23的版本，当应用集成进去之后，会自动使用oaid从而获取设备id，可以自行从OAID官网下载，也可以从此处下载[1.0.23](https://res.xdcdn.net/tapdb/Android/oaid/oaid_sdk_1.0.23.aar)的版本【如需其它版本请自行去oaid官网下载；然后将oaid sdk（下载下来是个aar)， 导入应用的工程目录


## 4.2、 数美sdk获取设备ID
下载[数美sdk](https://res.xdcdn.net/tapdb/Android/xdwl/xdwl-pri1-release.aar)导入应用的工程目录

**以下均是数美sdk要求的，详情可以参照[数美sdk官网Android部分](https://help.ishumei.com/docs/tw/sdk/Android)**

**在AndroidManifest.xml添加如下权限**
```
<!-- 必选权限 -->
<uses-permission android:name="android.permission.INTERNET" />
<uses-permission android:name="android.permission.READ_PHONE_STATE" />
<uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
<uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>
<uses-permission android:name="android.permission.READ_EXTERNAL_STORAGE"/>
<uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
<!-- 强烈建议权限 -->
<uses-permission android:name="android.permission.ACCESS_COARSE_LOCATION" />
```

**注意：
SDK 默认使用 HTTP 传输数据，在 targetSdkVersion >= 28 时需要在 AndroidManifest.xml 增加如下配置**

<application
    ...
    android:usesCleartextTraffic="true"
    ...

___
# 服务端部分补充
由于SDK无法推送准确的在线数据，这里提供服务端在线数据推送接口。游戏服务端可以每隔5分钟自行统计在线人数，通过接口推送到TapDB。TapDB进行数据汇总展现。

```
接口：https://se.tapdb.net/tapdb/online
方法：POST
格式：json
必需头信息：Content-Type: application/json
```

请求内容：

参数名 | 参数类型 | 参数说明
| ------ | ------ | ------ |
appid | string | 游戏的APP ID
onlines | array | 多条在线数据（最多100条）

其中onlines数组的结构为

参数名 | 参数类型 | 参数说明
| ------ | ------ | ------ |
server | string | 服务器。TapDB对同一服务器每一个自然5分钟仅接受一次数据
online | int | 在线人数
timestamp | long | 当前统计数据的时间戳(秒)。TapDB会按照自然5分钟进行数据对齐

示例：

```
{
  "appid":"gkjasd13bbsa1sdk",
  "onlines":[{
    "server":"s1",
    "online":123,
    "timestamp":1489739590
  },{
    "server":"s2",
    "online":188,
    "timestamp":1489739560
  }]
}
```

成功判断：返回的HTTP Code为200时认为发送成功，否则认为失败




